[![Build and run tests](https://github.com/GoodRequest/user-repository-sequelize/actions/workflows/build.yaml/badge.svg)](https://github.com/GoodRequest/user-repository-sequelize/actions/workflows/build.yaml)
[![Publish package to GitHub Packages](https://github.com/GoodRequest/user-repository-sequelize/actions/workflows/publish.yaml/badge.svg)](https://github.com/GoodRequest/user-repository-sequelize/actions/workflows/publish.yaml)

# User Repository implemented using Sequelize
This is implementation of user repository interface specified in [passport-jwt-wrapper](https://www.npmjs.com/package/@goodrequest/passport-jwt-wrapper) library.
It uses [sequelize](https://www.npmjs.com/package/sequelize) library for the data storage.

## Installation
`npm i --save @goodrequest/user-repository-sequelize`

## Usage
```typescript
import { UserRepository } from '@goodrequest/user-repository-sequelize'
import { models } from './db/models'

initAuth(passport, {
	userRepository: new UserRepository(models.User),
	refreshTokenRepository
})
```

### Models Templates
Library also exports basic Sequelize models:
- `BaseUserModel`: [BaseUserModel](src/models/baseUser.ts) - `id` is not defined in the attributes, needs to expanded
- `UUIDUserModel`: [UUIDUserModel](src/models/user-uuid.ts) - Uses `uuid` as `id`
- `BigIntUserModel`: [BigIntUserModel](src/models/user-int.ts)- Uses `BIGINT` with autoincrement as `id`

Model definition is divided into:
- `UserModelClass`
- `UserModelAttributes`
- `UserModelOptions`

These parts are used in the sequelize `init` method:
`UserModel.init(UserModelAttributes, UserModelOptions)`

Model needs to be defined and can be expanded using these parts. Examples are in the [tests](tests) folder:
- [BigIntUser](tests/db/models/int-user.ts)
- [UUIDUser](tests/db/models/uuid-user.ts):
```typescript
export class UuidUserModel extends UUIDUserModel {}

/* eslint-disable no-param-reassign */
export default (sequelize: Sequelize, modelName: string) => {
	UuidUserModel.init(
		{
			...UUIDUserModelAttributes,
			// overload createdBy column definition
			createdBy: {
				type: DataTypes.UUID,
				allowNull: true
			},
			// virtual attributes
			fullName: {
				type: DataTypes.VIRTUAL,
				get() {
					return `${this.name ?? ''} ${this.surname ?? ''}`.trim()
				}
			}
		},
		{
			...UUIDUserModelOptions,
			sequelize,
			modelName,
			tableName: 'uuid-users'
		}
	)

	return UuidUserModel
}
```

