"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BigIntUserModelOptions = exports.BigIntUserModelAttributes = exports.BigIntUserModel = void 0;
/* eslint-disable import/no-cycle */
const sequelize_1 = require("sequelize");
const baseUser_1 = require("./baseUser");
const helper_1 = require("../../tests/utils/helper");
class BigIntUserModel extends baseUser_1.BaseUserModel {
}
exports.BigIntUserModel = BigIntUserModel;
exports.BigIntUserModelAttributes = {
    ...baseUser_1.BaseUserModelAttributes,
    id: {
        type: sequelize_1.DataTypes.BIGINT,
        allowNull: false,
        primaryKey: true,
        autoIncrement: true
    },
    nameSlug: {
        type: sequelize_1.DataTypes.TEXT,
        allowNull: true
    },
    surname: {
        type: sequelize_1.DataTypes.TEXT,
        allowNull: true
    },
    surnameSlug: {
        type: sequelize_1.DataTypes.TEXT,
        allowNull: true
    },
    phone: {
        type: sequelize_1.DataTypes.TEXT,
        allowNull: true
    },
    phoneSlug: {
        type: sequelize_1.DataTypes.TEXT,
        allowNull: true
    },
    // foreign keys
    createdBy: {
        type: sequelize_1.DataTypes.BIGINT,
        allowNull: false
    },
    updatedBy: {
        type: sequelize_1.DataTypes.BIGINT,
        allowNull: true
    },
    deletedBy: {
        type: sequelize_1.DataTypes.BIGINT,
        allowNull: true
    }
};
/* eslint-disable no-param-reassign */
exports.BigIntUserModelOptions = {
    paranoid: true,
    timestamps: true,
    hooks: {
        beforeValidate: (user, options) => {
            if (user.changed('email')) {
                user.emailSlug = (0, helper_1.createSlug)(user.email);
                options.fields?.push('emailSlug');
            }
            if (user.changed('name')) {
                user.nameSlug = (0, helper_1.createSlug)(user.name);
                options.fields?.push('nameSlug');
            }
            if (user.changed('surname')) {
                user.surnameSlug = (0, helper_1.createSlug)(user.surname);
                options.fields?.push('surnameSlug');
            }
            if (user.changed('phone')) {
                user.phoneSlug = (0, helper_1.createSlug)(user.phone);
                options.fields?.push('phoneSlug');
            }
        },
        beforeCreate: (user) => {
            if (user.email) {
                user.emailSlug = (0, helper_1.createSlug)(user.email);
            }
            if (user.name) {
                user.nameSlug = (0, helper_1.createSlug)(user.name);
            }
            if (user.surname) {
                user.surnameSlug = (0, helper_1.createSlug)(user.surname);
            }
            if (user.phone) {
                user.phoneSlug = (0, helper_1.createSlug)(user.phone);
            }
        },
        beforeUpdate: (user) => {
            if (user.changed('name')) {
                user.nameSlug = (0, helper_1.createSlug)(user.name);
            }
            if (user.changed('surname')) {
                user.surnameSlug = (0, helper_1.createSlug)(user.surname);
            }
            if (user.changed('phone')) {
                user.phoneSlug = (0, helper_1.createSlug)(user.phone);
            }
        },
        beforeBulkCreate: (users) => {
            users.forEach((user) => {
                if (user.email) {
                    user.emailSlug = (0, helper_1.createSlug)(user.email);
                }
                if (user.name) {
                    user.nameSlug = (0, helper_1.createSlug)(user.name);
                }
                if (user.surname) {
                    user.surnameSlug = (0, helper_1.createSlug)(user.surname);
                }
                if (user.phone) {
                    user.phoneSlug = (0, helper_1.createSlug)(user.phone);
                }
            });
        },
        beforeBulkUpdate: (options) => {
            options.attributes = {
                ...options.attributes
            };
            if (options.attributes.name !== undefined) {
                options.attributes.nameSlug = (0, helper_1.createSlug)(options.attributes.name) || null;
                options.fields?.push('nameSlug');
            }
            if (options.attributes.surname !== undefined) {
                options.attributes.surnameSlug = (0, helper_1.createSlug)(options.attributes.surname) || null;
                options.fields?.push('surnameSlug');
            }
            if (options.attributes.company !== undefined) {
                options.attributes.companySlug = (0, helper_1.createSlug)(options.attributes.company) || null;
                options.fields?.push('companySlug');
            }
            if (options.attributes.companyRole !== undefined) {
                options.attributes.companyRoleSlug = (0, helper_1.createSlug)(options.attributes.companyRole) || null;
                options.fields?.push('companyRoleSlug');
            }
            if (options.attributes.phone !== undefined) {
                options.attributes.phoneSlug = (0, helper_1.createSlug)(options.attributes.phone) || null;
                options.fields?.push('phoneSlug');
            }
        }
    }
};
function defineUserModelInt() {
    return BigIntUserModel;
}
exports.default = defineUserModelInt();
