"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserRepository = void 0;
const sequelize_1 = require("sequelize");
const helpers_1 = require("./utils/helpers");
// eslint-disable-next-line import/prefer-default-export
class UserRepository {
    constructor(user) {
        this.user = user;
    }
    async getUsers(limit, page, sort, search) {
        const userConditions = {
            [sequelize_1.Op.and]: []
        };
        if (search) {
            const slugValue = (0, helpers_1.createSlug)(search);
            const searchConditions = slugValue.split('-').map((slug) => (0, sequelize_1.where)((0, sequelize_1.fn)('CONCAT_WS', '-', (0, sequelize_1.col)('user.emailSlug'), (0, sequelize_1.col)('user.nameSlug'), (0, sequelize_1.col)('user.surnameSlug'), (0, sequelize_1.col)('user.phoneSlug')), {
                [sequelize_1.Op.iLike]: `%${slug}%`
            }));
            userConditions[sequelize_1.Op.and].push(...searchConditions);
        }
        const offset = page * limit - limit;
        const users = await this.user.findAll({
            attributes: ['id', 'name', 'surname', 'email', 'permission', 'confirmedAt'],
            where: userConditions,
            offset,
            limit,
            order: [['id', 'ASC']]
        });
        return users;
    }
    async getUserByEmail(email) {
        const user = await this.user.findOne({
            attributes: ['id', 'name', 'surname', 'email', 'permission', 'confirmedAt', 'hash'],
            where: {
                email: {
                    [sequelize_1.Op.iLike]: email
                },
                confirmedAt: {
                    [sequelize_1.Op.not]: null
                },
                hash: {
                    [sequelize_1.Op.not]: null
                }
            }
        });
        return user ?? undefined;
    }
    async getUserById(id) {
        const user = await this.user.findByPk(id, {
            attributes: ['id', 'name', 'surname', 'email', 'permission', 'confirmedAt', 'hash']
        });
        return user ?? undefined;
    }
    async updateUserPassword(id, newPasswordHash) {
        const user = await this.user.findByPk(id, {
            attributes: ['id', 'hash']
        });
        if (!user) {
            throw new Error('error:User was not found');
        }
        const transaction = await this.user.sequelize?.transaction();
        try {
            const updatedUsers = await user.update({
                hash: newPasswordHash,
                updatedBy: id
            }, {
                transaction
            });
            if (transaction) {
                await transaction.commit();
            }
            return updatedUsers;
        }
        catch (e) {
            if (transaction) {
                await transaction.rollback();
            }
            throw e;
        }
    }
    /**
     * return user only if he has no password set -- is not registered
     * @param userID
     */
    async getNewUserById(userID) {
        const user = await this.user.findOne({
            where: {
                id: userID,
                hash: { [sequelize_1.Op.is]: null }
            }
        });
        return user ?? undefined;
    }
}
exports.UserRepository = UserRepository;
