"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const passport_jwt_wrapper_1 = require("@goodrequest/passport-jwt-wrapper");
const enums_1 = require("../../src/utils/enums");
const src_1 = require("../../src");
const models_1 = require("../db/models");
const declareTests_1 = require("../declareTests");
const helper_1 = require("../utils/helper");
const { UserInt } = models_1.models;
/* eslint-disable @typescript-eslint/no-unused-expressions */
describe(`Big Int User`, () => {
    const userRepo = new src_1.UserRepository(UserInt);
    describe('Good seeds', () => {
        const dbUsers = [];
        before(async () => {
            const hash = await (0, passport_jwt_wrapper_1.createHash)(helper_1.password);
            const mappedData = helper_1.userData.map((data) => ({
                email: data.email,
                confirmedAt: data.isConfirmed ? Date.now() : null,
                hash: data.isConfirmed ? hash : null,
                permission: data.permission
            }));
            const seedUsers = await UserInt.bulkCreate(mappedData);
            dbUsers.push(...seedUsers);
        });
        (0, declareTests_1.declareUserRepositoryTests)(userRepo, dbUsers);
    });
    describe('Wrong seeds', () => {
        beforeEach(async () => {
            await UserInt.truncate();
        });
        it('no seeds', async () => {
            const user = await userRepo.getUserById(1);
            (0, chai_1.expect)(user).not.to.exist;
        });
        it('`getUserById` More users with same email', async () => {
            const hash = await (0, passport_jwt_wrapper_1.createHash)(helper_1.password);
            const data = {
                email: 'user@goodrequest.com',
                permission: enums_1.PERMISSION.USER,
                confirmedAt: Date.now(),
                hash
            };
            const [seedUser1, seedUser2] = await UserInt.bulkCreate([
                data,
                {
                    ...data,
                    email: 'user@goodrequest.com'
                }
            ]);
            const user = await userRepo.getUserById(seedUser1.id);
            const user2 = await userRepo.getUserById(seedUser2.id);
            (0, chai_1.expect)(user).to.exist;
            (0, chai_1.expect)(user2).to.exist;
            (0, chai_1.expect)(user?.id).not.to.eq(user2?.id);
            (0, chai_1.expect)(user?.email).to.eq(user2?.email);
        });
        it('`getNewUserById` More users with same email', async function test() {
            if (!userRepo.getNewUserById) {
                return this.skip();
            }
            const data = {
                email: 'user@goodrequest.com',
                permission: enums_1.PERMISSION.USER
            };
            const [seedUser1, seedUser2] = await UserInt.bulkCreate([
                data,
                {
                    ...data,
                    email: 'user@goodrequest.com'
                }
            ]);
            const user = await userRepo.getNewUserById(seedUser1.id);
            const user2 = await userRepo.getNewUserById(seedUser2.id);
            (0, chai_1.expect)(user).to.exist;
            (0, chai_1.expect)(user2).to.exist;
            (0, chai_1.expect)(user?.id).not.to.eq(user2?.id);
            return (0, chai_1.expect)(user?.email).to.eq(user2?.email);
        });
        it('`getUserByEmail` More users with same email', async () => {
            const hash = await (0, passport_jwt_wrapper_1.createHash)(helper_1.password);
            const data = {
                email: 'user@goodrequest.com',
                permission: enums_1.PERMISSION.USER,
                confirmedAt: Date.now(),
                hash
            };
            const [seedUser1, seedUser2] = await UserInt.bulkCreate([
                data,
                {
                    ...data,
                    email: 'user@goodrequest.com'
                }
            ]);
            const user = await userRepo.getUserByEmail(seedUser1.email);
            const user2 = await userRepo.getUserByEmail(seedUser2.email);
            (0, chai_1.expect)(user).to.exist;
            (0, chai_1.expect)(user2).to.exist;
            (0, chai_1.expect)(user?.id).to.eq(user2?.id);
            (0, chai_1.expect)(user?.email).to.eq(user2?.email);
        });
    });
});
