"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mochaHooks = exports.mochaGlobalTeardown = exports.mochaGlobalSetup = void 0;
const sequelize_1 = require("sequelize");
const database_1 = require("./config/database");
const indexModels_1 = require("./db/models/indexModels");
const uuid_user_1 = require("./db/models/uuid-user");
const int_user_1 = require("./db/models/int-user");
const connection = new sequelize_1.Sequelize(`${database_1.testMainDB.baseUrl}/postgres`, database_1.testMainDB.options);
const mochaGlobalSetup = async () => {
    try {
        const seq = new sequelize_1.Sequelize(database_1.testMainDB.url, database_1.testMainDB.options);
        await Promise.all([connection.authenticate(), seq.authenticate()]);
        const models = (0, indexModels_1.modelsBuilder)(seq);
        uuid_user_1.UuidUserModel.belongsTo(models.UserUUID, { as: 'creator', foreignKey: 'createdBy' });
        uuid_user_1.UuidUserModel.belongsTo(models.UserUUID, { as: 'editor', foreignKey: 'updatedBy' });
        uuid_user_1.UuidUserModel.belongsTo(models.UserUUID, { as: 'destructor', foreignKey: 'deletedBy' });
        int_user_1.IntUserModel.belongsTo(models.UserInt, { as: 'creator', foreignKey: 'createdBy' });
        int_user_1.IntUserModel.belongsTo(models.UserInt, { as: 'editor', foreignKey: 'updatedBy' });
        int_user_1.IntUserModel.belongsTo(models.UserInt, { as: 'destructor', foreignKey: 'deletedBy' });
        // sync db
        await seq.sync({ force: true });
        await seq.close();
        return await Promise.resolve();
    }
    catch (err) {
        console.log('mochaGlobalSetup error: ', err);
        return Promise.reject(err);
    }
};
exports.mochaGlobalSetup = mochaGlobalSetup;
async function mochaGlobalTeardown() {
    try {
        // remove test databases after all test finished
        const [testDatabases] = await connection.query(`select datname from pg_database where datname like '${database_1.testDatabaseName}_%'`);
        await Promise.all(testDatabases.map((testDatabase) => connection.query(/* SQL */ `DROP DATABASE IF EXISTS ${testDatabase.datname} WITH (FORCE);`)));
        await connection.close();
        return await Promise.resolve();
    }
    catch (err) {
        console.log('mochaGlobalTeardown error: ', err);
        return Promise.reject(err);
    }
}
exports.mochaGlobalTeardown = mochaGlobalTeardown;
const mochaHooks = async () => {
    // create jwt access tokens
    return {
        async beforeAll() {
            try {
                const isUnitTest = this.test.parent.suites.every((el) => /unit.test/.test(el.file));
                if (!isUnitTest) {
                    // drop test database for worker
                    await connection.query(/* SQL */ `DROP DATABASE IF EXISTS ${database_1.testDatabaseWorkerName} WITH (FORCE);`);
                    // create test database for worker
                    await connection.query(/* SQL */ `CREATE DATABASE ${database_1.testDatabaseWorkerName} TEMPLATE ${database_1.testDatabaseName};`);
                }
                return await Promise.resolve();
            }
            catch (err) {
                console.log('beforeAll error: ', err);
                return Promise.reject(err);
            }
        },
        async afterAll() {
            try {
                return await Promise.resolve();
            }
            catch (err) {
                console.log('afterAll error: ', err);
                return Promise.reject(err);
            }
        }
    };
};
exports.mochaHooks = mochaHooks;
