import { DataTypes, Model } from 'sequelize'
import { ID } from '@goodrequest/passport-jwt-wrapper'
import { PERMISSION, PERMISSIONS } from '../utils/enums'

export class BaseUserModel<IDTYPE extends ID> extends Model {
	declare id: IDTYPE
	declare email: string
	declare emailSlug: string
	declare hash: string
	declare permission: PERMISSION
	declare confirmedAt: string
	// metadata
	declare createdAt: string
	declare updatedAt: string
	declare deletedAt: string
}

/**
 * Does not have id defined
 */
export const BaseUserModelAttributes = {
	email: {
		type: DataTypes.TEXT,
		allowNull: false
	},
	emailSlug: {
		type: DataTypes.TEXT,
		allowNull: false
	},
	name: {
		type: DataTypes.TEXT,
		allowNull: true
	},
	hash: {
		type: DataTypes.TEXT,
		allowNull: true
	},
	permission: {
		type: DataTypes.ENUM(...PERMISSIONS),
		allowNull: false
	},
	confirmedAt: {
		type: DataTypes.DATE,
		allowNull: true
	}
}

function defineBaseUserModel() {
	return BaseUserModel
}

export default defineBaseUserModel()
