// eslint-disable-next-line import/no-extraneous-dependencies
import pg from 'pg'
import { Sequelize } from 'sequelize'

import * as database from '../../config/database'

// types
import { IRestoreHookFunctions } from '../../types/models'

import { modelsBuilder } from './indexModels'
import { UuidUserModel } from './uuid-user'
import { IntUserModel } from './int-user'

// NOTE: set true because otherwise BIGINT return string instead of integer https://github.com/sequelize/sequelize/issues/1774
pg.defaults.parseInt8 = true

// eslint-disable-next-line import/namespace
const { url, options: dbOptions } = database.test

if (dbOptions.logging) {
	dbOptions.logging = (log: string) => {
		console.log(log, { language: 'sql', ignoreIllegals: true, theme: 'code' })
	}
}

const sequelize = <Sequelize & IRestoreHookFunctions>new Sequelize(url, dbOptions)
sequelize
	.authenticate()
	.then(() => console.log('Database connection has been established successfully'))
	.catch((err: any) => console.log(`Unable to connect to the database: ${err}`))

const buildModels = () => {
	const models = modelsBuilder(sequelize)

	UuidUserModel.belongsTo(models.UserUUID, { as: 'creator', foreignKey: 'createdBy' })
	UuidUserModel.belongsTo(models.UserUUID, { as: 'editor', foreignKey: 'updatedBy' })
	UuidUserModel.belongsTo(models.UserUUID, { as: 'destructor', foreignKey: 'deletedBy' })

	IntUserModel.belongsTo(models.UserInt, { as: 'creator', foreignKey: 'createdBy' })
	IntUserModel.belongsTo(models.UserInt, { as: 'editor', foreignKey: 'updatedBy' })
	IntUserModel.belongsTo(models.UserInt, { as: 'destructor', foreignKey: 'deletedBy' })

	return models
}

const models = buildModels()
type Models = typeof models

export { models }
export type { Models }
export default sequelize
