import { DataTypes, Sequelize } from 'sequelize'

import { UUIDUserModelAttributes, UUIDUserModel } from '../../../src'
import { UUIDUserModelOptions } from '../../../src/models/user-uuid'

export class UuidUserModel extends UUIDUserModel {}

/* eslint-disable no-param-reassign */
export default (sequelize: Sequelize, modelName: string) => {
	UuidUserModel.init(
		{
			...UUIDUserModelAttributes,
			// overload createdBy column definition
			createdBy: {
				type: DataTypes.UUID,
				allowNull: true
			},
			// virtual attributes
			fullName: {
				type: DataTypes.VIRTUAL,
				get() {
					return `${this.name ?? ''} ${this.surname ?? ''}`.trim()
				}
			}
		},
		{
			...UUIDUserModelOptions,
			sequelize,
			modelName,
			tableName: 'uuid-users'
		}
	)

	return UuidUserModel
}
